#include <Adafruit_GFX.h>     // Biblioteca gráfica principal
#include <XTronical_ST7735.h> // Biblioteca específica do display
#include <SPI.h>              // Biblioteca de comunicação SPI

// Configura os pinos que serão usados para a comunicação com o display
#define TFT_DC 2        // Seleção do registro
#define TFT_RST 4       // Pino de redefinição da tela
#define TFT_CS 5        // ativação do Display, se não ativado, não comunicará no barramento SPI


// Inicia a rotina de conversa com o display com os pinos configurados 
// Como não colocamos o TFT_SCLK e o TFT_MOSI, a rotina presume que estamos usando o barramento SPI
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  

//Instancia as Tasks
TaskHandle_t Task1, Task2;

//Constantes e variáveis -----------------------------------------------------------------

const int RST = 25;              // Porta digital D25 - Reset do DRV8825
const int SLP = 26;              // Porta digital D26 - Dormir (Sleep) DRV8825
const int ENA = 34;              // Porta digital D34 - Ativa (enable) DRV8825
const int M0 = 35;               // Porta digital D35 - M0 do DRV8825
const int M1 = 32;               // Porta digital D32 - M1 do DRV8825
const int M2 = 33;               // Porta digital D33 - M2 do DRV8825
const int DIR = 14;              // Porta digital D14 - Direção (direction) do DRV8825
const int STP = 27;              // Porta digital D27 - Passo (Step) do DRV8825
const int POTD = 13;             // AD do potenciometro do driver
const int POTV = 12;             // AD do potenciometro de velocidade do motor

int MeioPeriodo;                 // Meio periodo do pulso em microsegundos 
float MMP;                       // Multiplicador de micropasso
int PPR = 200;                   // Número de passos por volta
int Pulsos;                      // Pulsos para o driver do motor
int Voltas;                      // voltas do motor
float RPM;                       // Rotacao por minuto
int leituraAd;                   // Valor da leitura do potenciômetro do driver
float tensaoPot;                 // Valor da tensão na saída do potenciômetro do driver
float correnteMot;               // Valor da corrente configurada no driver

// Funções ---------------------------------------------------------------------------------

// Imprime os valores das variáveis no Display
void Imprime(){
  
  // Cria um retângulo preto em cima dos dados impressos para apagá-los
  tft.fillRect(24, 0, 48, 16, ST7735_BLACK);
  // Posiciona o cursor na posição
  tft.setCursor(24,0);
  // Imprime os dados na posição do cursor
  tft.print(int(tensaoPot));
  
  tft.fillRect(24, 20, 48, 16, ST7735_BLACK);
  tft.setCursor(24,20);
  tft.print(correnteMot);
  
  tft.fillRect(48, 40, 80, 16, ST7735_BLACK);
  tft.setCursor(48,40);
  tft.print(int(RPM));

  tft.fillRect(36, 60, 48, 16, ST7735_BLACK);
  tft.setCursor(36,60);
  tft.print(analogRead(POTD));

  tft.fillRect(0, 100, 48, 16, ST7735_BLACK);
  tft.setCursor(0,100);
  tft.print(MeioPeriodo);
 
}

// Calcula a corrente máxima suportada pelo driver
//Os dados dessa equação foram tiradas do datasheet do DRV8825
float calculoCorrente(float tensao) {
  return (tensao / 1000) / (5 * 0.1);   // 0.1: valor do resistor do driver 
}

//Calcula o RPM do motor
float calculoRPM() {
  //TPV: tempo por volta(segundos), PPR: passo por revolucao, MMP: multiplicador de micro passo
  float TPV = (PPR * MMP * (MeioPeriodo * 2)) / 1000000;
  //converte o TPV em RPM      
  return ((1.0 / TPV) * 60.0);                                
}
//-----------------------------------------------------------------------------------------
void setup() {

  //Inicia a comunicação serial
  Serial.begin(115200);
  
  // Configura as portas como saída
  pinMode(RST, OUTPUT);       
  pinMode(SLP, OUTPUT);
  pinMode(ENA, OUTPUT);
  pinMode(M0, OUTPUT);
  pinMode(M1, OUTPUT);
  pinMode(M2, OUTPUT);
  pinMode(DIR, OUTPUT);
  pinMode(STP, OUTPUT);
  // Configura as portas como entrada
  pinMode(POTD, INPUT);     
  pinMode(POTV, INPUT);

  //Configuração do driver----------------------------------------------------------------

  // MMP: Multiplicador de micro passo:
  // 1- passo inteiro
  // 2- meio passo
  // 4- 1/4 de passo
  // 8- 1/8 de passo
  // 16 - 1/16 de passo
  // 32 - 1/32 de passo
  MMP = 1;

  // Configura as portas do driver para passo inteiro                  
  digitalWrite(M0, LOW);
  digitalWrite(M1, LOW);
  digitalWrite(M2, LOW);
  // Desativa o modo sleep
  digitalWrite(SLP, HIGH);
  // Desativa o modo reset
  digitalWrite(RST, HIGH);
  // Ativa as saídas do driver
  digitalWrite(ENA, LOW);
  // Define a direção do motor
  digitalWrite(DIR, HIGH);

//Configuração do Display ---------------------------------------------------------------------

  // Inicializa o Display
  tft.init();  

  delay(100);

  // Gira a tela em 180 graus
  tft.setRotation(2);
  // Define o tamanho do texto
  tft.setTextSize(2);
  // Define a cor do texto
  tft.setTextColor(ST7735_WHITE);
  // Define a quebra de linha do texto
  tft.setTextWrap(false);
  // Pinta a tela de preto
  tft.fillScreen(ST7735_BLACK);

  delay(500);
  
  // Imprime as informações estáticas na tela
  tft.setCursor(0,0);
  tft.print("T:");
  tft.setCursor(72,0);
  tft.print("mv");

  tft.setCursor(0,20);
  tft.print("C:");
  tft.setCursor(72,20);
  tft.print("A");
  
  tft.setCursor(0,40);
  tft.print("RPM:");

  tft.setCursor(0,60);
  tft.print("AD:");

  tft.setCursor(48,100);
  // 229 é o código da letra "µ" na tabela ASCII
  tft.print((char)229);
  tft.print("s");

  delay(500);

  // Configuração das tasks
  xTaskCreatePinnedToCore(
    codeForTask1,       // Função Task
    "led1Task",         // Nome da Task
    2000,               // Tamanho da pilha da Task 
    NULL,               // Parâmetro da Task
    1,                  // Prioridade da Task
    &Task1,             // Identificação da Task para acompanhar a Task criada
    0);                 // Core usado

  //Delay necessário para iniciar a task 1
  delay(500);

  xTaskCreatePinnedToCore(
    codeForTask2,
    "led2Task",
    2000,
    NULL,
    1,
    &Task2,
    1);

    delay(500);
}// Fim do setup

//----------------------------------------------------------------------------------------
// Task 1 ---------------------------------------------------------------------------------
void codeForTask1( void * parameter )
{
  for (;;) {

    // Lê o valor do potenciômetro de velocidade
    MeioPeriodo = (analogRead(POTV)) / 2;
    // Lê o valor do potenciômetro do driver
    leituraAd = analogRead(POTD);
    // Calcula a tensão da saída do potenciômetro do driver 
    tensaoPot = 0.7492*leituraAd + 185.92;
    //Calcula a corrente máxima suportada pelo driver
    correnteMot = calculoCorrente(tensaoPot);
    //Calcula o RPM do motor
    RPM = calculoRPM();
    //Imprime as informações no display
    Imprime();

    delay(100);
    
  }
}
// Task 2 ---------------------------------------------------------------------------------
void codeForTask2( void * parameter )
{
  for (;;) {
    //Aqui é gerado o pulso do motor
    digitalWrite(STP, LOW);                   // Pulso nível baixo
    delayMicroseconds(MeioPeriodo);           // MeioPeriodo de X microsegundos
    digitalWrite(STP, HIGH);                  // Pulso nível alto
    delayMicroseconds (MeioPeriodo);          // MeioPeriodo de X microsegundos
  }
}

// A função void loop() não será usada pelo programa
// Porém é necessária para funcionar corretamente
void loop() {
  
  delay(100);

}
